/***
 * Excerpted from "Stripes: and Java Web Development is Fun Again",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material, 
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/fdstr for more book information.
***/
package stripesbook.action;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import net.sourceforge.stripes.action.DefaultHandler;
import net.sourceforge.stripes.action.DontValidate;
import net.sourceforge.stripes.action.ForwardResolution;
import net.sourceforge.stripes.action.Resolution;
import net.sourceforge.stripes.action.SimpleMessage;
import net.sourceforge.stripes.ajax.JavaScriptResolution;
import net.sourceforge.stripes.validation.EmailTypeConverter;
import net.sourceforge.stripes.validation.Validate;
import net.sourceforge.stripes.validation.ValidateNestedProperties;
import net.sourceforge.stripes.validation.ValidationError;
import net.sourceforge.stripes.validation.ValidationErrorHandler;
import net.sourceforge.stripes.validation.ValidationErrors;
import stripesbook.model.Contact;
import stripesbook.model.Gender;

public class ServerFormActionBean extends BaseActionBean
    implements ValidationErrorHandler
{
    private static final String FORM =
        "/WEB-INF/jsp/server_side_validation.jsp";

    @DefaultHandler
    public Resolution form() {
        return new ForwardResolution(FORM);
    }
    public Resolution save() {
        getContext().getMessages()
            .add(new SimpleMessage("All input is valid."));
        return new ForwardResolution(FORM);
    }
    
    public Resolution validate() {
        return new JavaScriptResolution(Collections.emptyMap());
    }
    
    @DontValidate
    public Resolution cancel() {
        getContext().getMessages()
            .add(new SimpleMessage("Action cancelled."));
        return new ForwardResolution(FORM);
    }
    
    public Resolution handleValidationErrors(ValidationErrors errors) {
        String header =
            getContext().getRequest().getHeader("X-Requested-With");

        if (header != null && header.equalsIgnoreCase("XMLHttpRequest")) {
            Map<String,List<String>> map = new HashMap<String,List<String>>();
            Locale locale = getContext().getLocale();
            for (String key : errors.keySet()) {
                List<ValidationError> errorList = errors.get(key);
                List<String> messages = new ArrayList<String>(errorList.size());
                for (ValidationError error : errorList) {
                    messages.add(error.getMessage(locale));
                }
                map.put(key, messages);
            }
            return new JavaScriptResolution(map);
        }
        return null;
    }
    

    @ValidateNestedProperties({
        @Validate(field="firstName", maxlength=25),
        @Validate(field="lastName",  minlength=2, maxlength=40),
        @Validate(field="email", required=true, on={"save","validate"},
            converter=EmailTypeConverter.class),
        @Validate(field="birthDate", expression="${this < today}")
    })
    private Contact contact;
    public Contact getContact() {
        return contact;
    }
    public void setContact(Contact contact) {
        this.contact = contact;
    }
    public Date getToday() {
        return new Date();
    }
    public Gender[] getGenders() {
        return Gender.values();
    }
}
