/***
 * Excerpted from "Stripes: and Java Web Development is Fun Again",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material, 
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/fdstr for more book information.
***/

package stripesbook.dao.impl.stripersist;


import org.stripesstuff.stripersist.Stripersist;

import java.io.Serializable;
import java.lang.reflect.ParameterizedType;
import java.util.List;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import stripesbook.dao.Dao;
import stripesbook.model.User;


import static org.hibernate.criterion.Restrictions.*;



public abstract class BaseDaoImpl<T,ID extends Serializable>
    implements Dao<T,ID>
{
    /* other methods... */


    private Class<T> entityClass;

    @SuppressWarnings("unchecked")
    public BaseDaoImpl() {
        entityClass = (Class<T>)
            ((ParameterizedType) getClass().getGenericSuperclass())
            .getActualTypeArguments()[0];
    }
    @SuppressWarnings("unchecked")
    public List<T> read() {
        return getSession().createCriteria(entityClass).list();
    }
    public T read(ID id) {
        return Stripersist.getEntityManager().find(entityClass, id);
    }
    @SuppressWarnings("unchecked")
    public void save(T object) {
        Stripersist.getEntityManager().persist(object);
    }
    public void delete(T object) {
        Stripersist.getEntityManager().remove(object);
    }
    public void commit() {
        Stripersist.getEntityManager().getTransaction().commit();
    }

    @SuppressWarnings("unchecked")
    public T findBy(String fieldName, Object value) {
        Criteria criteria = getSession().createCriteria(entityClass)
            .add(eq(fieldName, value));
        return getSingleResult(criteria);
    }
    @SuppressWarnings("unchecked")
    public T findBy(String fieldName, Object value, User user) {
        Criteria criteria = getSession().createCriteria(entityClass)
            .add(and(
                eq(fieldName, value),
                eq("user", user)
            ));
        return getSingleResult(criteria);
    }
    @SuppressWarnings("unchecked")
    private T getSingleResult(Criteria criteria) {
        try {
            return (T) criteria.uniqueResult();
        }
        catch (HibernateException exc) {
            return (T) criteria.list().get(0);
        }
    }

    
    protected Session getSession() {
        return (Session) Stripersist.getEntityManager().getDelegate();
    }
    

}

