/***
 * Excerpted from "Stripes: and Java Web Development is Fun Again",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material, 
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/fdstr for more book information.
***/

package stripesbook.test.stripesmock;


import java.util.HashMap;
import java.util.Map;
import net.sourceforge.stripes.controller.DispatcherServlet;
import net.sourceforge.stripes.controller.StripesFilter;
import net.sourceforge.stripes.mock.MockHttpSession;
import net.sourceforge.stripes.mock.MockRoundtrip;
import net.sourceforge.stripes.mock.MockServletContext;
import org.junit.BeforeClass;
import org.junit.Test;
import stripesbook.action.ContactFormActionBean;
import stripesbook.action.LoginActionBean;
import stripesbook.action.MockDataLoaderActionBean;
import stripesbook.model.PhoneNumber;

import static org.junit.Assert.*;


public class ContactFormActionBeanTest {
    private static MockServletContext mockServletContext;
    private static MockHttpSession mockSession;

    @BeforeClass
    public static void setup() throws Exception {
        mockServletContext = new MockServletContext("webmail");

        Map<String,String> params = new HashMap<String,String>();
        params.put("ActionResolver.Packages", "stripesbook.action");
        params.put("Extension.Packages", "stripesbook.ext,"
            + "org.stripesstuff.stripersist");
        mockServletContext.addFilter(StripesFilter.class, /* (1) */
            "StripesFilter", params);

        mockServletContext.setServlet(DispatcherServlet.class,
            "DispatcherServlet", null);

        mockSession = new MockHttpSession(mockServletContext); /* (2) */
        

        
        // Load mock user
        MockRoundtrip trip = new MockRoundtrip(mockServletContext,
            MockDataLoaderActionBean.class, mockSession);
        trip.execute();

        // Login mock user
        trip = new MockRoundtrip(mockServletContext,
            LoginActionBean.class, mockSession);
        trip.setParameter("username", "freddy");
        trip.setParameter("password", "nadia");
        trip.execute("login");
        
    
    }
    
    
    @Test
    public void testEmailRequired() throws Exception {
        MockRoundtrip trip = new MockRoundtrip(mockServletContext,
            ContactFormActionBean.class, mockSession);

        trip.execute("save");

        ContactFormActionBean bean =
            trip.getActionBean(ContactFormActionBean.class);

        assertEquals(1,
            bean.getContext().getValidationErrors().size());

        assertEquals(MockRoundtrip.DEFAULT_SOURCE_PAGE,
            trip.getDestination());
    }
    
    
    @Test
    public void testSaveValid() throws Exception {
        MockRoundtrip trip = new MockRoundtrip(mockServletContext,
            ContactFormActionBean.class, mockSession);

        trip.setParameter("contact.email", "test@test.com");
        trip.setParameter("contact.phoneNumber", "654-456-4567");
        trip.execute("save");

        ContactFormActionBean bean =
            trip.getActionBean(ContactFormActionBean.class);

        assertEquals(0,
            bean.getContext().getValidationErrors().size());

        PhoneNumber pn = bean.getContact().getPhoneNumber();
        assertEquals("654", pn.getAreaCode());
        assertEquals("456", pn.getPrefix());
        assertEquals("4567", pn.getSuffix());

        assertTrue(
            trip.getDestination().startsWith("/ContactList.action"));
    }
    
}

