/***
 * Excerpted from "Stripes: and Java Web Development is Fun Again",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material, 
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/fdstr for more book information.
***/
package stripesbook.dao.mock;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import stripesbook.dao.Dao;
import stripesbook.model.ModelBase;

/**
 * Mock abstract implementation of the DAO. This implementation
 * does not attempt to be efficient; rather, it serves the purposes of
 * providing a simple working DAO with no dependencies on libraries or
 * a database.
 */
public abstract class MockDao<T extends ModelBase>
    implements Dao<T,Integer>
{
    private List<T> list = new ArrayList<T>();
    private Map<Integer,T> map = new HashMap<Integer,T>();
    private int nextId = 1;

    public void save(T object) {
        if (object != null) {
            if (object.getId() == null) {
                int id = nextId++;
                object.setId(id);
                list.add(object);
            }
            else {
                Integer id = object.getId();
                list.set(list.indexOf(map.get(id)), object);
            }
            map.put(object.getId(), object);
        }
    }

    public List<T> read() {
        return list;
    }

    public T read(Integer id) {
        return map.get(id);
    }

    public void delete(T object) {
        list.remove(object);
        map.remove(object.getId());
    }

    public void commit() {
    }
}
