/***
 * Excerpted from "Stripes: and Java Web Development is Fun Again",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material, 
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/fdstr for more book information.
***/

package stripesbook.test.plainmock;


import java.util.Locale;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import net.sourceforge.stripes.localization.LocalePicker;
import org.junit.Before;
import org.junit.Test;
import stripesbook.ext.MyLocalePicker;


import static org.junit.Assert.*;
import static org.mockito.Mockito.*;

public class MyLocalePickerTest {
    private LocalePicker localePicker;
    private HttpServletRequest req;
    private HttpSession session;

    @Before
    public void setup() {
        localePicker = new MyLocalePicker();
        req = mock(HttpServletRequest.class);
        session = mock(HttpSession.class);
        stub(req.getSession()).toReturn(session);
    }


    @Test
    public void testLocaleFrInRequest() {
        stub(req.getParameter(MyLocalePicker.LOCALE)).toReturn("fr");
        Locale locale = localePicker.pickLocale(req);
        assertEquals(Locale.FRENCH, locale);
    }


    @Test
    public void testLocaleFrInSession() {
        stub(session.getAttribute(MyLocalePicker.LOCALE)).toReturn("fr");
        Locale locale = localePicker.pickLocale(req);
        assertEquals(Locale.FRENCH, locale);
    }


    @Test
    public void testLocaleInRequestOverridesSession() {
        stub(session.getAttribute(MyLocalePicker.LOCALE)).toReturn("fr");
        stub(req.getParameter(MyLocalePicker.LOCALE)).toReturn("en");
        Locale locale = localePicker.pickLocale(req);
        assertEquals(Locale.ENGLISH, locale);
    }


}

